@@ -39,7 +39,10 @@
 #ifndef __MLME_H__
 #define __MLME_H__
 
-//extern UCHAR BROADCAST_ADDR[];
+#include "rtmp_dot11.h"
+
+#ifdef CONFIG_STA_SUPPORT
+#endif // CONFIG_STA_SUPPORT //
 
 // maximum supported capability information - 
 // ESS, IBSS, Privacy, Short Preamble, Spectrum mgmt, Short Slot
@@ -61,27 +64,24 @@
 #define JAP_W56	4
 #define MAX_RD_REGION 5
 
-#ifdef	NDIS51_MINIPORT
-#define BEACON_LOST_TIME            4000       // 2048 msec = 2 sec
-#else
 #define BEACON_LOST_TIME            4 * OS_HZ    // 2048 msec = 2 sec
-#endif
 
 #define DLS_TIMEOUT                 1200      // unit: msec
 #define AUTH_TIMEOUT                300       // unit: msec
 #define ASSOC_TIMEOUT               300       // unit: msec
-#define JOIN_TIMEOUT                2 * OS_HZ      // unit: msec
+#define JOIN_TIMEOUT                2000        // unit: msec
 #define SHORT_CHANNEL_TIME          90        // unit: msec
 #define MIN_CHANNEL_TIME            110        // unit: msec, for dual band scan
 #define MAX_CHANNEL_TIME            140       // unit: msec, for single band scan
 #define	FAST_ACTIVE_SCAN_TIME	    30 		  // Active scan waiting for probe response time
 #define CW_MIN_IN_BITS              4         // actual CwMin = 2^CW_MIN_IN_BITS - 1
+#define AUTO_CHANNEL_SEL_TIMEOUT		400		// uint: msec
+#define LINK_DOWN_TIMEOUT           20000      // unit: msec
+#define AUTO_WAKEUP_TIMEOUT			70			//unit: msec
 
 
 #ifdef CONFIG_STA_SUPPORT
-#ifndef CONFIG_AP_SUPPORT
 #define CW_MAX_IN_BITS              10        // actual CwMax = 2^CW_MAX_IN_BITS - 1
-#endif
 #endif // CONFIG_STA_SUPPORT //
 
 #ifdef CONFIG_APSTA_MIXED_SUPPORT
@@ -145,9 +145,9 @@ extern UINT32 CW_MAX_IN_BITS;
 //#define BSS_TABLE_EMPTY(x)             ((x).BssNr == 0)
 #define MAC_ADDR_IS_GROUP(Addr)       (((Addr[0]) & 0x01))
 #define MAC_ADDR_HASH(Addr)            (Addr[0] ^ Addr[1] ^ Addr[2] ^ Addr[3] ^ Addr[4] ^ Addr[5])
-#define MAC_ADDR_HASH_INDEX(Addr)      (MAC_ADDR_HASH(Addr) % HASH_TABLE_SIZE)
+#define MAC_ADDR_HASH_INDEX(Addr)      (MAC_ADDR_HASH(Addr) & (HASH_TABLE_SIZE - 1))
 #define TID_MAC_HASH(Addr,TID)            (TID^Addr[0] ^ Addr[1] ^ Addr[2] ^ Addr[3] ^ Addr[4] ^ Addr[5])
-#define TID_MAC_HASH_INDEX(Addr,TID)      (TID_MAC_HASH(Addr,TID) % HASH_TABLE_SIZE)
+#define TID_MAC_HASH_INDEX(Addr,TID)      (TID_MAC_HASH(Addr,TID) & (HASH_TABLE_SIZE - 1))
 
 // LED Control
 // assoiation ON. one LED ON. another blinking when TX, OFF when idle
@@ -212,11 +212,57 @@ if (((__pEntry)) != NULL) \
 	(__pEntry)->OneSecTxNoRetryOkCount = 0; \
 }
 
+//#ifdef RTMP_RBUS_SUPPORT
+#ifdef NEW_RATE_ADAPT_SUPPORT
+//added ys
+//added for rate adaptation by ys
+//#define NEW_RATE_ADAPT_SUPPORT//3T3R always use the new rate adaptation algorithm, regardless whether this is defined
+//#define CCK_SUPPORT
+//#define USE_NEW_THRD//not useful when (NEW_RATE_ADAPT_SUPPORT is not defined) and (2T2R)
+#define USE_GREATER_UP_MCS //rate(upMcs)> rate(thisMcs) if this is defined. otherwise, rate(upMcs)>= rate(thisMcs); meaningful when (NEW_RATE_ADAPT_SUPPORT is supported) or 3T3R
+#define PER_THRD_ADJ			1
+#define RA_PER_LOW_THRD			8
+#define FEW_PKTS_CNT_THRD 1
+//#define RA_PER_HIGH_THRD_FACTOR 90 //high thrd = 1 - (rate of lower MCS) * factor/100/(rate of this MCS), useful when NEW_RATE_ADAPT_SUPPORT and USE_NEW_THRD are defined
+
+//#if !defined(NEW_RATE_ADAPT_SUPPORT)
+//	#undef USE_NEW_THRD//actually, whether this is defined doesn't matter when !defined(NEW_RATE_ADAPT_SUPPORT). This is just for clarification.
+//	#undef USE_GREATER_UP_MCS //actually, whether this is defined doesn't matter when !defined(NEW_RATE_ADAPT_SUPPORT). This is just for clarification.
+//#endif
+//#define MAX_STREAMS 2, this information is now determined by pAd->MACVersion >= RALINK_2883_VERSION
+//#if MAX_STREAMS==3
+//	#define NEW_RATE_ADAPT_SUPPORT
+//#endif
+#endif // NEW_RATE_ADAPT_SUPPORT //
+//#endif // RTMP_RBUS_SUPPORT //
+
+#ifdef TXBF_SUPPORT
+//#define MRQ_FORCE_TX//regardless the capability of the station
+#define ETXBF_EN_COND		0 //this value can be set by iwpriv ra0 set ETxBfEnCond=?
+// 0:no etxbf, 
+// 1:etxbf update periodically, 
+// 2:etxbf updated if mcs changes in RateSwitchingAdapt() or APQuickResponeForRateUpExecAdapt(). 
+// 3:auto-selection: if mfb changes or timer expires, then send sounding packets <----not finished yet!!!
+// note: when = 1 or 3, NO_SNDG_CNT_THRD controls the frequency to update the matrix(ETXBF_EN_COND=1) or activate the whole bf evaluation process(not defined)
+
+#define MSI_TOGGLE_BF		6
+#define TOGGLE_BF_PKTS		5// the number of packets with inverted BF status
+#define READY_FOR_SNDG0		0//jump to WAIT_SNDG_FB0 when channel change or periodically
+#define WAIT_SNDG_FB0		1//jump to WAIT_SNDG_FB1 when bf report0 is received
+#define WAIT_SNDG_FB1		2
+#define WAIT_MFB			3
+#define WAIT_USELESS_RSP	4
+#define WAIT_BEST_SNDG		5
+#define NO_SNDG_CNT_THRD	0//send sndg packet if there is no sounding for (NO_SNDG_CNT_THRD+1)*500msec. If this =0, bf matrix is updated at each call of APMlmeDynamicTxRateSwitchingAdapt()
+//rate adaptation end
+#endif // TXBF_SUPPORT //
+
+
 //
 // 802.11 frame formats
 //
 //  HT Capability INFO field in HT Cap IE .   
-typedef struct PACKED {
+typedef struct GNU_PACKED {
 #ifdef RT_BIG_ENDIAN
 	USHORT	LSIGTxopProSup:1;
 	USHORT	Forty_Mhz_Intolerant:1;
@@ -251,7 +297,7 @@ typedef struct PACKED {
 } HT_CAP_INFO, *PHT_CAP_INFO;
 
 //  HT Capability INFO field in HT Cap IE .   
-typedef struct PACKED {
+typedef struct GNU_PACKED {
 #ifdef RT_BIG_ENDIAN
 	UCHAR	rsv:3;//momi power safe
 	UCHAR	MpduDensity:3;
@@ -264,7 +310,7 @@ typedef struct PACKED {
 } HT_CAP_PARM, *PHT_CAP_PARM;
 
 //  HT Capability INFO field in HT Cap IE .   
-typedef struct PACKED {
+typedef struct GNU_PACKED {
 	UCHAR	MCSSet[10];
 	UCHAR	SupRate[2];  // unit : 1Mbps
 #ifdef RT_BIG_ENDIAN
@@ -284,7 +330,7 @@ typedef struct PACKED {
 } HT_MCS_SET, *PHT_MCS_SET;
 
 //  HT Capability INFO field in HT Cap IE .  
-typedef struct PACKED {
+typedef struct GNU_PACKED {
 #ifdef RT_BIG_ENDIAN 
 	USHORT	rsv2:4;
 	USHORT	RDGSupport:1;	//reverse Direction Grant  support
@@ -305,7 +351,7 @@ typedef struct PACKED {
 } EXT_HT_CAP_INFO, *PEXT_HT_CAP_INFO;
 
 //  HT Beamforming field in HT Cap IE .   
-typedef struct PACKED _HT_BF_CAP{
+typedef struct GNU_PACKED _HT_BF_CAP{
 #ifdef RT_BIG_ENDIAN
 	ULONG	rsv:3;
 	ULONG	ChanEstimation:2;
@@ -352,7 +398,7 @@ typedef struct PACKED _HT_BF_CAP{
 } HT_BF_CAP, *PHT_BF_CAP;
 
 //  HT antenna selection field in HT Cap IE .   
-typedef struct PACKED _HT_AS_CAP{
+typedef struct GNU_PACKED _HT_AS_CAP{
 #ifdef RT_BIG_ENDIAN
 	UCHAR	rsv:1;
 	UCHAR	TxSoundPPDU:1;
@@ -377,7 +423,7 @@ typedef struct PACKED _HT_AS_CAP{
 // Draft 1.0 set IE length 26, but is extensible..
 #define SIZE_HT_CAP_IE		26
 // The structure for HT Capability IE.
-typedef struct PACKED _HT_CAPABILITY_IE{
+typedef struct GNU_PACKED _HT_CAPABILITY_IE{
 	HT_CAP_INFO		HtCapInfo;
 	HT_CAP_PARM		HtCapParm;
 //	HT_MCS_SET		HtMCSSet;
@@ -395,13 +441,13 @@ typedef struct PACKED _HT_CAPABILITY_IE{
 #define dot11BSSWidthTriggerScanInterval					300  // in sec. max interval between scan operations to be performed to detect BSS channel width trigger events.
 #define dot11OBSSScanPassiveTotalPerChannel					200	// in TU. min total amount of time that the STA scans each channel when performing a passive OBSS scan.
 #define dot11OBSSScanActiveTotalPerChannel					20	//in TU. min total amount of time that the STA scans each channel when performing a active OBSS scan
-#define dot11BSSWidthChannelTransactionDelayFactor			5	// min ratio between the delay time in performing a switch from 20MHz BSS to 20/40 BSS operation and the maxima
+#define dot11BSSWidthChannelTransactionDelayFactor			5	// min ratio between the delay time in performing a switch from 20MHz BSS to 20/40 BSS operation and the maximum
 																//	interval between overlapping BSS scan operations.
 #define dot11BSSScanActivityThreshold						25	// in %%, max total time that a STA may be active on the medium during a period of 
 																//	(dot11BSSWidthChannelTransactionDelayFactor * dot11BSSWidthTriggerScanInterval) seconds without
 																//	being obligated to perform OBSS Scan operations. default is 25(== 0.25%)
 
-typedef struct PACKED _OVERLAP_BSS_SCAN_IE{
+typedef struct GNU_PACKED _OVERLAP_BSS_SCAN_IE{
 	USHORT		ScanPassiveDwell;
 	USHORT		ScanActiveDwell;
 	USHORT		TriggerScanInt;				// Trigger scan interval
@@ -413,10 +459,12 @@ typedef struct PACKED _OVERLAP_BSS_SCAN_IE{
 
 
 //  7.3.2.56. 20/40 Coexistence element used in  Element ID = 72 = IE_2040_BSS_COEXIST
-typedef union PACKED _BSS_2040_COEXIST_IE{
- struct PACKED {
+typedef union GNU_PACKED _BSS_2040_COEXIST_IE{
+ struct GNU_PACKED {
  #ifdef RT_BIG_ENDIAN
-	UCHAR	rsv:5;
+	UCHAR	rsv:3;
+ 	UCHAR	ObssScanExempGrant:1;
+	UCHAR	ObssScanExempReq:1;
 	UCHAR	BSS20WidthReq:1;
 	UCHAR	Intolerant40:1;	
 	UCHAR	InfoReq:1; 
@@ -424,7 +472,9 @@ typedef union PACKED _BSS_2040_COEXIST_IE{
 	UCHAR	InfoReq:1;
 	UCHAR	Intolerant40:1;			// Inter-BSS. set 1 when prohibits a receiving BSS from operating as a 20/40 Mhz BSS.
 	UCHAR	BSS20WidthReq:1;		// Intra-BSS set 1 when prohibits a receiving AP from operating its BSS as a 20/40MHz BSS.
-	UCHAR	rsv:5;
+	UCHAR	ObssScanExempReq:1;
+	UCHAR	ObssScanExempGrant:1;
+	UCHAR	rsv:3;
 #endif // RT_BIG_ENDIAN //
     } field;
  UCHAR   word;
@@ -436,7 +486,6 @@ typedef struct  _TRIGGER_EVENTA{
 	UCHAR	BSSID[6];	
 	UCHAR	RegClass;	// Regulatory Class
 	USHORT	Channel;
-	ULONG	CDCounter;   // Maintain a seperate count down counter for each Event A.
 } TRIGGER_EVENTA, *PTRIGGER_EVENTA;
 
 // 20/40 trigger event table
@@ -450,7 +499,7 @@ typedef struct  _TRIGGER_EVENT_TAB{
 
 // 7.3.27 20/40 Bss Coexistence Mgmt capability used in extended capabilities information IE( ID = 127 = IE_EXT_CAPABILITY). 
 //	This is the first octet and was defined in 802.11n D3.03 and 802.11yD9.0
-typedef struct PACKED _EXT_CAP_INFO_ELEMENT{
+typedef struct GNU_PACKED _EXT_CAP_INFO_ELEMENT{
 #ifdef RT_BIG_ENDIAN
 	UCHAR	rsv2:5;
 	UCHAR	ExtendChannelSwitch:1;
@@ -466,7 +515,7 @@ typedef struct PACKED _EXT_CAP_INFO_ELEMENT{
 
 
 // 802.11n 7.3.2.61
-typedef struct PACKED _BSS_2040_COEXIST_ELEMENT{
+typedef struct GNU_PACKED _BSS_2040_COEXIST_ELEMENT{
 	UCHAR					ElementID;		// ID = IE_2040_BSS_COEXIST = 72
 	UCHAR					Len;
 	BSS_2040_COEXIST_IE		BssCoexistIe;
@@ -474,7 +523,7 @@ typedef struct PACKED _BSS_2040_COEXIST_ELEMENT{
 
 
 //802.11n 7.3.2.59
-typedef struct PACKED _BSS_2040_INTOLERANT_CH_REPORT{
+typedef struct GNU_PACKED _BSS_2040_INTOLERANT_CH_REPORT{
 	UCHAR				ElementID;		// ID = IE_2040_BSS_INTOLERANT_REPORT = 73
 	UCHAR				Len;	
 	UCHAR				RegulatoryClass;
@@ -483,7 +532,7 @@ typedef struct PACKED _BSS_2040_INTOLERANT_CH_REPORT{
 
 
 // The structure for channel switch annoucement IE. This is in 802.11n D3.03
-typedef struct PACKED _CHA_SWITCH_ANNOUNCE_IE{
+typedef struct GNU_PACKED _CHA_SWITCH_ANNOUNCE_IE{
 	UCHAR			SwitchMode;	//channel switch mode
 	UCHAR			NewChannel;	// 
 	UCHAR			SwitchCount;	// 
@@ -491,7 +540,7 @@ typedef struct PACKED _CHA_SWITCH_ANNOUNCE_IE{
 
 
 // The structure for channel switch annoucement IE. This is in 802.11n D3.03
-typedef struct PACKED _SEC_CHA_OFFSET_IE{
+typedef struct GNU_PACKED _SEC_CHA_OFFSET_IE{
 	UCHAR			SecondaryChannelOffset;	 // 1: Secondary above, 3: Secondary below, 0: no Secondary 
 } SEC_CHA_OFFSET_IE, *PSEC_CHA_OFFSET_IE;
 
@@ -501,18 +550,12 @@ typedef struct {
 	BOOLEAN			bHtEnable;	 // If we should use ht rate.
 	BOOLEAN			bPreNHt;	 // If we should use ht rate.
 	//Substract from HT Capability IE
-	UCHAR			MCSSet[16];	//only supoort MCS=0-15,32 , 
+	UCHAR			MCSSet[16];
 } RT_HT_PHY_INFO, *PRT_HT_PHY_INFO;
 
 //This structure substracts ralink supports from all 802.11n-related features.
 //Features not listed here but contained in 802.11n spec are not supported in rt2860.
 typedef struct {
-#if 0	// move to 
-	BOOLEAN			bHtEnable;	 // If we should use ht rate.
-	BOOLEAN			bPreNHt;	 // If we should use ht rate.
-	//Substract from HT Capability IE
-	UCHAR			MCSSet[16];	//only supoort MCS=0-15,32 , 
-#endif	
 #ifdef RT_BIG_ENDIAN
 	USHORT	rsv:5;
 	USHORT	AmsduSize:1;	// Max receiving A-MSDU size
@@ -571,7 +614,7 @@ typedef struct {
 } RT_HT_CAPABILITY, *PRT_HT_CAPABILITY;
 
 //   field in Addtional HT Information IE .   
-typedef struct PACKED {
+typedef struct GNU_PACKED {
 #ifdef RT_BIG_ENDIAN
 	UCHAR	SerInterGranu:3;
 	UCHAR	S_PSMPSup:1;
@@ -587,7 +630,7 @@ typedef struct PACKED {
 #endif
 } ADD_HTINFO, *PADD_HTINFO;
 
-typedef struct PACKED{
+typedef struct GNU_PACKED{
 #ifdef RT_BIG_ENDIAN
 	USHORT	rsv2:11; 
 	USHORT	OBSS_NonHTExist:1;
@@ -605,7 +648,7 @@ typedef struct PACKED{
 
 
 // TODO: Need sync with spec about the definition of StbcMcs. In Draft 3.03, it's reserved.
-typedef struct PACKED{
+typedef struct GNU_PACKED{
 #ifdef RT_BIG_ENDIAN
 	USHORT	rsv:4;
 	USHORT	PcoPhase:1;
@@ -628,7 +671,7 @@ typedef struct PACKED{
 } ADD_HTINFO3, *PADD_HTINFO3;
 
 #define SIZE_ADD_HT_INFO_IE		22
-typedef struct  PACKED{
+typedef struct  GNU_PACKED{
 	UCHAR				ControlChan;
 	ADD_HTINFO			AddHtInfo;
 	ADD_HTINFO2			AddHtInfo2;	 
@@ -636,128 +679,30 @@ typedef struct  PACKED{
 	UCHAR				MCSSet[16];		// Basic MCS set
 } ADD_HT_INFO_IE, *PADD_HT_INFO_IE;
 
-typedef struct  PACKED{
+typedef struct  GNU_PACKED{
 	UCHAR				NewExtChanOffset;
 } NEW_EXT_CHAN_IE, *PNEW_EXT_CHAN_IE;
 
-
-// 4-byte HTC field.  maybe included in any frame except non-QOS data frame.  The Order bit must set 1.
-typedef struct PACKED {
-#ifdef RT_BIG_ENDIAN
-    UINT32		RDG:1;	//RDG / More PPDU
-    UINT32		ACConstraint:1;	//feedback request
-    UINT32		rsv:5;  //calibration sequence
-    UINT32		ZLFAnnouce:1;	// ZLF announcement
-    UINT32		CSISTEERING:2;	//CSI/ STEERING
-    UINT32		FBKReq:2;	//feedback request
-    UINT32		CalSeq:2;  //calibration sequence
-    UINT32		CalPos:2;	// calibration position
-    UINT32		MFBorASC:7;	//Link adaptation feedback containing recommended MCS. 0x7f for no feedback or not available
-    UINT32		MFS:3;	//SET to the received value of MRS. 0x111 for unsolicited MFB.
-    UINT32		MRSorASI:3;	// MRQ Sequence identifier. unchanged during entire procedure. 0x000-0x110.
-    UINT32		MRQ:1;	//MCS feedback. Request for a MCS feedback
-    UINT32		TRQ:1;	//sounding request
-    UINT32		MA:1;	//management action payload exist in (QoS Null+HTC)
-#else
-    UINT32		MA:1;	//management action payload exist in (QoS Null+HTC)
-    UINT32		TRQ:1;	//sounding request
-    UINT32		MRQ:1;	//MCS feedback. Request for a MCS feedback
-    UINT32		MRSorASI:3;	// MRQ Sequence identifier. unchanged during entire procedure. 0x000-0x110.
-    UINT32		MFS:3;	//SET to the received value of MRS. 0x111 for unsolicited MFB.
-    UINT32		MFBorASC:7;	//Link adaptation feedback containing recommended MCS. 0x7f for no feedback or not available
-    UINT32		CalPos:2;	// calibration position
-    UINT32		CalSeq:2;  //calibration sequence
-    UINT32		FBKReq:2;	//feedback request
-    UINT32		CSISTEERING:2;	//CSI/ STEERING
-    UINT32		ZLFAnnouce:1;	// ZLF announcement
-    UINT32		rsv:5;  //calibration sequence
-    UINT32		ACConstraint:1;	//feedback request
-    UINT32		RDG:1;	//RDG / More PPDU
-#endif /* !RT_BIG_ENDIAN */
-} HT_CONTROL, *PHT_CONTROL;
-
-// 2-byte QOS CONTROL field
-typedef struct PACKED {
-#ifdef RT_BIG_ENDIAN
-    USHORT      Txop_QueueSize:8;
-    USHORT      AMsduPresent:1;
-    USHORT      AckPolicy:2;  //0: normal ACK 1:No ACK 2:scheduled under MTBA/PSMP  3: BA
-    USHORT      EOSP:1;
-    USHORT      TID:4;
-#else
-    USHORT      TID:4;
-    USHORT      EOSP:1;
-    USHORT      AckPolicy:2;  //0: normal ACK 1:No ACK 2:scheduled under MTBA/PSMP  3: BA
-    USHORT      AMsduPresent:1;
-    USHORT      Txop_QueueSize:8;
-#endif /* !RT_BIG_ENDIAN */
-} QOS_CONTROL, *PQOS_CONTROL;	
-
-// 2-byte Frame control field
-typedef	struct	PACKED {
-#ifdef RT_BIG_ENDIAN
-	USHORT		Order:1;			// Strict order expected
-	USHORT		Wep:1;				// Wep data
-	USHORT		MoreData:1;			// More data bit
-	USHORT		PwrMgmt:1;			// Power management bit
-	USHORT		Retry:1;			// Retry status bit
-	USHORT		MoreFrag:1;			// More fragment bit
-	USHORT		FrDs:1;				// From DS indication
-	USHORT		ToDs:1;				// To DS indication
-	USHORT		SubType:4;			// MSDU subtype
-	USHORT		Type:2;				// MSDU type
-	USHORT		Ver:2;				// Protocol version
-#else
-	USHORT		Ver:2;				// Protocol version
-	USHORT		Type:2;				// MSDU type
-	USHORT		SubType:4;			// MSDU subtype
-	USHORT		ToDs:1;				// To DS indication
-	USHORT		FrDs:1;				// From DS indication
-	USHORT		MoreFrag:1;			// More fragment bit
-	USHORT		Retry:1;			// Retry status bit
-	USHORT		PwrMgmt:1;			// Power management bit
-	USHORT		MoreData:1;			// More data bit
-	USHORT		Wep:1;				// Wep data
-	USHORT		Order:1;			// Strict order expected
-#endif /* !RT_BIG_ENDIAN */
-} FRAME_CONTROL, *PFRAME_CONTROL;
-
-typedef	struct	PACKED _HEADER_802_11	{
-    FRAME_CONTROL   FC;
-    USHORT          Duration;
-    UCHAR           Addr1[MAC_ADDR_LEN];
-    UCHAR           Addr2[MAC_ADDR_LEN];
-	UCHAR			Addr3[MAC_ADDR_LEN];
-#ifdef RT_BIG_ENDIAN
-	USHORT			Sequence:12;
-	USHORT			Frag:4;
-#else
-	USHORT			Frag:4;
-	USHORT			Sequence:12;
-#endif /* !RT_BIG_ENDIAN */
-	UCHAR			Octet[0];
-}	HEADER_802_11, *PHEADER_802_11;
-
-typedef struct PACKED _FRAME_802_11 {
+typedef struct GNU_PACKED _FRAME_802_11 {
     HEADER_802_11   Hdr;
     UCHAR            Octet[1];
 }   FRAME_802_11, *PFRAME_802_11;
 
 // QoSNull embedding of management action. When HT Control MA field set to 1.
-typedef struct PACKED _MA_BODY {
+typedef struct GNU_PACKED _MA_BODY {
     UCHAR            Category;
     UCHAR            Action;
     UCHAR            Octet[1];
 }   MA_BODY, *PMA_BODY;
 
-typedef	struct	PACKED _HEADER_802_3	{
+typedef	struct	GNU_PACKED _HEADER_802_3	{
     UCHAR           DAAddr1[MAC_ADDR_LEN];
     UCHAR           SAAddr2[MAC_ADDR_LEN];
     UCHAR           Octet[2];
 }	HEADER_802_3, *PHEADER_802_3;
 ////Block ACK related format
 // 2-byte BA Parameter  field  in 	DELBA frames to terminate an already set up bA
-typedef struct PACKED{
+typedef struct GNU_PACKED{
 #ifdef RT_BIG_ENDIAN
     USHORT      TID:4;	// value of TC os TS
     USHORT      Initiator:1;	// 1: originator    0:recipient
@@ -770,7 +715,7 @@ typedef struct PACKED{
 } DELBA_PARM, *PDELBA_PARM;
 
 // 2-byte BA Parameter Set field  in ADDBA frames to signal parm for setting up a BA
-typedef struct PACKED {
+typedef struct GNU_PACKED {
 #ifdef RT_BIG_ENDIAN
     USHORT      BufSize:10;	// number of buffe of size 2304 octetsr
     USHORT      TID:4;	// value of TC os TS
@@ -785,8 +730,8 @@ typedef struct PACKED {
 } BA_PARM, *PBA_PARM;
 
 // 2-byte BA Starting Seq CONTROL field
-typedef union PACKED {
-    struct PACKED {
+typedef union GNU_PACKED {
+    struct GNU_PACKED {
 #ifdef RT_BIG_ENDIAN
     USHORT      StartSeq:12;   // sequence number of the 1st MSDU for which this BAR is sent
 	USHORT      FragNum:4;	// always set to 0
@@ -800,7 +745,7 @@ typedef union PACKED {
 
 //BAControl and BARControl are the same
 // 2-byte BA CONTROL field in BA frame
-typedef struct PACKED {
+typedef struct GNU_PACKED {
 #ifdef RT_BIG_ENDIAN
     USHORT      TID:4;
     USHORT      Rsv:9;
@@ -817,7 +762,7 @@ typedef struct PACKED {
 } BA_CONTROL, *PBA_CONTROL;
 
 // 2-byte BAR CONTROL field in BAR frame
-typedef struct PACKED {
+typedef struct GNU_PACKED {
 #ifdef RT_BIG_ENDIAN
     USHORT      TID:4;
     USHORT      Rsv1:9;
@@ -834,7 +779,7 @@ typedef struct PACKED {
 } BAR_CONTROL, *PBAR_CONTROL;
 
 // BARControl in MTBAR frame
-typedef struct PACKED {
+typedef struct GNU_PACKED {
 #ifdef RT_BIG_ENDIAN
     USHORT      NumTID:4;
     USHORT      Rsv1:9;
@@ -850,7 +795,7 @@ typedef struct PACKED {
 #endif /* !RT_BIG_ENDIAN */
 } MTBAR_CONTROL, *PMTBAR_CONTROL;
 
-typedef struct PACKED {
+typedef struct GNU_PACKED {
 #ifdef RT_BIG_ENDIAN
     USHORT      TID:4;
     USHORT      Rsv1:12;
@@ -866,22 +811,8 @@ typedef struct {
 } EACH_TID, *PEACH_TID;
 
 
-typedef struct PACKED _PSPOLL_FRAME {
-    FRAME_CONTROL   FC;
-    USHORT          Aid;
-    UCHAR           Bssid[MAC_ADDR_LEN];
-    UCHAR           Ta[MAC_ADDR_LEN];
-}   PSPOLL_FRAME, *PPSPOLL_FRAME;
-
-typedef	struct	PACKED _RTS_FRAME	{
-    FRAME_CONTROL   FC;
-    USHORT          Duration;
-    UCHAR           Addr1[MAC_ADDR_LEN];
-    UCHAR           Addr2[MAC_ADDR_LEN];
-}RTS_FRAME, *PRTS_FRAME;
-
 // BAREQ AND MTBAREQ have the same subtype BAR, 802.11n BAR use compressed bitmap.
-typedef struct PACKED _FRAME_BA_REQ {
+typedef struct GNU_PACKED _FRAME_BA_REQ {
 	FRAME_CONTROL   FC;
 	USHORT          Duration;
 	UCHAR           Addr1[MAC_ADDR_LEN];
@@ -890,7 +821,7 @@ typedef struct PACKED _FRAME_BA_REQ {
 	BASEQ_CONTROL 	 BAStartingSeq;
 }   FRAME_BA_REQ, *PFRAME_BA_REQ;
 
-typedef struct PACKED _FRAME_MTBA_REQ {
+typedef struct GNU_PACKED _FRAME_MTBA_REQ {
 	FRAME_CONTROL   FC;
 	USHORT          Duration;
 	UCHAR           Addr1[MAC_ADDR_LEN];
@@ -901,7 +832,7 @@ typedef struct PACKED _FRAME_MTBA_REQ {
 }   FRAME_MTBA_REQ, *PFRAME_MTBA_REQ;
 
 // Compressed format is mandantory in HT STA
-typedef struct PACKED _FRAME_MTBA {
+typedef struct GNU_PACKED _FRAME_MTBA {
 	FRAME_CONTROL   FC;
 	USHORT          Duration;
 	UCHAR           Addr1[MAC_ADDR_LEN];
@@ -911,14 +842,14 @@ typedef struct PACKED _FRAME_MTBA {
 	UCHAR		BitMap[8];
 }   FRAME_MTBA, *PFRAME_MTBA;
 
-typedef struct PACKED _FRAME_PSMP_ACTION {
+typedef struct GNU_PACKED _FRAME_PSMP_ACTION {
 	HEADER_802_11   Hdr;
 	UCHAR	Category;
 	UCHAR	Action;
 	UCHAR	Psmp;	// 7.3.1.25
 }   FRAME_PSMP_ACTION, *PFRAME_PSMP_ACTION;
 
-typedef struct PACKED _FRAME_ACTION_HDR {
+typedef struct GNU_PACKED _FRAME_ACTION_HDR {
 	HEADER_802_11   Hdr;
 	UCHAR	Category;
 	UCHAR	Action;
@@ -926,7 +857,7 @@ typedef struct PACKED _FRAME_ACTION_HDR {
 
 //Action Frame
 //Action Frame  Category:Spectrum,  Action:Channel Switch. 7.3.2.20
-typedef struct PACKED _CHAN_SWITCH_ANNOUNCE {
+typedef struct GNU_PACKED _CHAN_SWITCH_ANNOUNCE {
 	UCHAR					ElementID;	// ID = IE_CHANNEL_SWITCH_ANNOUNCEMENT = 37
 	UCHAR					Len;
 	CHA_SWITCH_ANNOUNCE_IE	CSAnnounceIe;
@@ -934,14 +865,14 @@ typedef struct PACKED _CHAN_SWITCH_ANNOUNCE {
 
 
 //802.11n : 7.3.2.20a
-typedef struct PACKED _SECOND_CHAN_OFFSET {
+typedef struct GNU_PACKED _SECOND_CHAN_OFFSET {
 	UCHAR				ElementID;		// ID = IE_SECONDARY_CH_OFFSET = 62
 	UCHAR				Len;
 	SEC_CHA_OFFSET_IE	SecChOffsetIe;
 }   SECOND_CHAN_OFFSET, *PSECOND_CHAN_OFFSET;
 
 
-typedef struct PACKED _FRAME_SPETRUM_CS {
+typedef struct GNU_PACKED _FRAME_SPETRUM_CS {
 	HEADER_802_11   Hdr;
 	UCHAR	Category;
 	UCHAR	Action;
@@ -950,7 +881,7 @@ typedef struct PACKED _FRAME_SPETRUM_CS {
 }   FRAME_SPETRUM_CS, *PFRAME_SPETRUM_CS;
 
 
-typedef struct PACKED _FRAME_ADDBA_REQ {
+typedef struct GNU_PACKED _FRAME_ADDBA_REQ {
 	HEADER_802_11   Hdr;
 	UCHAR	Category;
 	UCHAR	Action;
@@ -960,7 +891,7 @@ typedef struct PACKED _FRAME_ADDBA_REQ {
 	BASEQ_CONTROL	BaStartSeq; // 0-0
 }   FRAME_ADDBA_REQ, *PFRAME_ADDBA_REQ;
 
-typedef struct PACKED _FRAME_ADDBA_RSP {
+typedef struct GNU_PACKED _FRAME_ADDBA_RSP {
 	HEADER_802_11   Hdr;
 	UCHAR	Category;
 	UCHAR	Action;
@@ -970,7 +901,7 @@ typedef struct PACKED _FRAME_ADDBA_RSP {
 	USHORT		TimeOutValue;
 }   FRAME_ADDBA_RSP, *PFRAME_ADDBA_RSP;
 
-typedef struct PACKED _FRAME_DELBA_REQ {
+typedef struct GNU_PACKED _FRAME_DELBA_REQ {
 	HEADER_802_11   Hdr;
 	UCHAR	Category;
 	UCHAR	Action;
@@ -980,7 +911,7 @@ typedef struct PACKED _FRAME_DELBA_REQ {
 
 
 //7.2.1.7
-typedef struct PACKED _FRAME_BAR {
+typedef struct GNU_PACKED _FRAME_BAR {
 	FRAME_CONTROL   FC;
 	USHORT          Duration;
 	UCHAR           Addr1[MAC_ADDR_LEN];
@@ -990,7 +921,7 @@ typedef struct PACKED _FRAME_BAR {
 }   FRAME_BAR, *PFRAME_BAR;
 
 //7.2.1.7
-typedef struct PACKED _FRAME_BA {
+typedef struct GNU_PACKED _FRAME_BA {
 	FRAME_CONTROL   FC;
 	USHORT          Duration;
 	UCHAR           Addr1[MAC_ADDR_LEN];
@@ -1002,7 +933,7 @@ typedef struct PACKED _FRAME_BA {
 
 
 // Radio Measuement Request Frame Format
-typedef struct PACKED _FRAME_RM_REQ_ACTION {
+typedef struct GNU_PACKED _FRAME_RM_REQ_ACTION {
 	HEADER_802_11   Hdr;
 	UCHAR	Category;
 	UCHAR	Action;
@@ -1011,7 +942,7 @@ typedef struct PACKED _FRAME_RM_REQ_ACTION {
 	UCHAR   data[0];
 }   FRAME_RM_REQ_ACTION, *PFRAME_RM_REQ_ACTION;
 
-typedef struct PACKED {
+typedef struct GNU_PACKED {
 	UCHAR		ID;
 	UCHAR		Length;
 	UCHAR		ChannelSwitchMode;
@@ -1034,7 +965,7 @@ typedef struct PACKED {
 //
 // Contention-free parameter (without ID and Length)
 //
-typedef struct PACKED {
+typedef struct GNU_PACKED {
     BOOLEAN     bValid;         // 1: variable contains valid value
     UCHAR       CfpCount;
     UCHAR       CfpPeriod;
@@ -1076,7 +1007,7 @@ typedef struct {
 } QBSS_LOAD_PARM, *PQBSS_LOAD_PARM;
 
 // QBSS Info field in QSTA's assoc req
-typedef struct PACKED {
+typedef struct GNU_PACKED {
 #ifdef RT_BIG_ENDIAN
 	UCHAR		Rsv2:1;
 	UCHAR		MaxSPLength:2;
@@ -1096,8 +1027,17 @@ typedef struct PACKED {
 #endif /* !RT_BIG_ENDIAN */
 } QBSS_STA_INFO_PARM, *PQBSS_STA_INFO_PARM;
 
+typedef struct {
+	QBSS_STA_INFO_PARM	QosInfo;
+	UCHAR	Rsv;
+	UCHAR	Q_AC_BE[4];
+	UCHAR	Q_AC_BK[4];
+	UCHAR	Q_AC_VI[4];
+	UCHAR	Q_AC_VO[4];
+} QBSS_STA_EDCA_PARM, *PQBSS_STA_EDCA_PARM;
+
 // QBSS Info field in QAP's Beacon/ProbeRsp
-typedef struct PACKED {
+typedef struct GNU_PACKED {
 #ifdef RT_BIG_ENDIAN
 	UCHAR		UAPSD:1;
 	UCHAR		Rsv:3;
@@ -1144,8 +1084,10 @@ typedef struct {
 	UCHAR			HtCapabilityLen;
 	ADD_HT_INFO_IE AddHtInfo;	// AP might use this additional ht info IE 
 	UCHAR			AddHtInfoLen;
+	EXT_CAP_INFO_ELEMENT ExtCapInfo;	// this is the extened capibility IE appreed in MGMT frames. Doesn't need to update once set in Init.
 	UCHAR			NewExtChanOffset;
 	CHAR    Rssi;
+	CHAR	MinSNR;	
     UCHAR   Privacy;			// Indicate security function ON/OFF. Don't mess up with auth mode.
 	UCHAR	Hidden;
 
@@ -1174,6 +1116,8 @@ typedef struct {
 	NDIS_802_11_WEP_STATUS	WepStatus;				// Unicast Encryption Algorithm extract from VAR_IE
 	USHORT					VarIELen;				// Length of next VIE include EID & Length
 	UCHAR					VarIEs[MAX_VIE_LEN];
+	USHORT					VarIeFromProbeRspLen;
+	UCHAR					*pVarIeFromProbRsp;
 
 	// CCX Ckip information
     UCHAR   CkipFlag;
@@ -1186,14 +1130,28 @@ typedef struct {
 	EDCA_PARM           EdcaParm;
 	QOS_CAPABILITY_PARM QosCapability;
 	QBSS_LOAD_PARM      QbssLoad;
+
+
 #ifdef CONFIG_STA_SUPPORT
     WPA_IE_     WpaIE;
     WPA_IE_     RsnIE;
+	WPA_IE_ 	WpsIE;
+
 #ifdef EXT_BUILD_CHANNEL_LIST
 	UCHAR		CountryString[3];
 	BOOLEAN		bHasCountryIE;
 #endif // EXT_BUILD_CHANNEL_LIST //
 #endif // CONFIG_STA_SUPPORT //
+
+	UCHAR   MacAddr[MAC_ADDR_LEN];
+
+#ifdef LINUX
+#ifdef CONFIG_STA_SUPPORT
+#ifdef RT_CFG80211_SUPPORT
+	VOID *pCfg80211_Chan;
+#endif // RT_CFG80211_SUPPORT //
+#endif // CONFIG_STA_SUPPORT //
+#endif // LINUX //
 } BSS_ENTRY, *PBSS_ENTRY;
 
 typedef struct {
@@ -1204,10 +1162,10 @@ typedef struct {
 
 
 typedef struct _MLME_QUEUE_ELEM {
+	UCHAR             Msg[MGMT_DMA_BUFFER_SIZE];	// move here to fix alignment issue for ARM CPU
     ULONG             Machine;
     ULONG             MsgType;
     ULONG             MsgLen;
-    UCHAR             Msg[MGMT_DMA_BUFFER_SIZE];
     LARGE_INTEGER     TimeStamp;
     UCHAR             Rssi0;
     UCHAR             Rssi1;
@@ -1216,6 +1174,7 @@ typedef struct _MLME_QUEUE_ELEM {
     UCHAR             Channel;
     UCHAR             Wcid;
     BOOLEAN           Occupied;
+	ULONG             Priv;
 } MLME_QUEUE_ELEM, *PMLME_QUEUE_ELEM;
 
 typedef struct _MLME_QUEUE {
@@ -1270,6 +1229,7 @@ typedef struct _MLME_AUX {
 	HT_CAPABILITY_IE		HtCapability;
 	UCHAR		        	HtCapabilityLen;
 	ADD_HT_INFO_IE		AddHtInfo;	// AP might use this additional ht info IE 
+	EXT_CAP_INFO_ELEMENT ExtCapInfo; // this is the extened capibility IE appreed in MGMT frames. Doesn't need to update once set in Init.
 	UCHAR			NewExtChannelOffset;
 	//RT_HT_CAPABILITY	SupportedHtPhy;
 
@@ -1291,6 +1251,9 @@ typedef struct _MLME_AUX {
     RALINK_TIMER_STRUCT BeaconTimer, ScanTimer;
     RALINK_TIMER_STRUCT AuthTimer;
     RALINK_TIMER_STRUCT AssocTimer, ReassocTimer, DisassocTimer;
+
+#ifdef CONFIG_STA_SUPPORT
+#endif // CONFIG_STA_SUPPORT //
 } MLME_AUX, *PMLME_AUX;
 
 typedef struct _MLME_ADDBA_REQ_STRUCT{
@@ -1375,13 +1338,13 @@ typedef struct _MLME_DLS_REQ_STRUCT {
 #endif // QOS_DLS_SUPPORT //
 #endif // CONFIG_STA_SUPPORT //
 
-typedef struct PACKED {
+typedef struct GNU_PACKED {
     UCHAR   Eid;
     UCHAR   Len;
-    CHAR   Octet[1];
+    UCHAR   Octet[1];
 } EID_STRUCT,*PEID_STRUCT, BEACON_EID_STRUCT, *PBEACON_EID_STRUCT;
 
-typedef struct PACKED _RTMP_TX_RATE_SWITCH
+typedef struct GNU_PACKED _RTMP_TX_RATE_SWITCH
 {
 	UCHAR   ItemNo;
 #ifdef RT_BIG_ENDIAN
@@ -1404,6 +1367,36 @@ typedef struct PACKED _RTMP_TX_RATE_SWITCH
 	UCHAR   TrainDown;
 } RRTMP_TX_RATE_SWITCH, *PRTMP_TX_RATE_SWITCH;
 
+//#ifdef RTMP_RBUS_SUPPORT
+typedef struct  _RTMP_TX_RATE_SWITCH_3S
+{
+	UCHAR   ItemNo;
+#ifdef RT_BIG_ENDIAN
+	UCHAR	Rsv2:2;
+	UCHAR	Mode:2;
+	UCHAR	Rsv1:1;	
+	UCHAR	BW:1;
+	UCHAR	ShortGI:1;
+	UCHAR	STBC:1;
+#else
+	UCHAR	STBC:1;
+	UCHAR	ShortGI:1;
+	UCHAR	BW:1;
+	UCHAR	Rsv1:1;
+	UCHAR	Mode:2;
+	UCHAR	Rsv2:2;
+#endif	
+	UCHAR   CurrMCS;
+	UCHAR   TrainUp;
+	UCHAR   TrainDown;
+	UCHAR	downMcs;
+	UCHAR	upMcs3;
+	UCHAR	upMcs2;
+	UCHAR	upMcs1;
+	UCHAR	dataRate;
+} RRTMP_TX_RATE_SWITCH_3S, *PRTMP_TX_RATE_SWITCH_3S;
+//#endif // RTMP_RBUS_SUPPORT //
+
 // ========================== AP mlme.h ===============================
 #define TBTT_PRELOAD_TIME       384        // usec. LomgPreamble + 24-byte at 1Mbps
 #define DEFAULT_DTIM_PERIOD     1
@@ -1414,11 +1407,12 @@ typedef struct PACKED _RTMP_TX_RATE_SWITCH
 //#define RX_WEIGHTING                     60
 
 #define MAC_TABLE_AGEOUT_TIME			300			// unit: sec
+#define MAC_TABLE_MIN_AGEOUT_TIME		60			// unit: sec
 #define MAC_TABLE_ASSOC_TIMEOUT			5			// unit: sec
 #define MAC_TABLE_FULL(Tab)				((Tab).size == MAX_LEN_OF_MAC_TABLE)
 
 // AP shall drop the sta if contine Tx fail count reach it.
-#define MAC_ENTRY_LIFE_CHECK_CNT		20			// packet cnt.
+#define MAC_ENTRY_LIFE_CHECK_CNT		1024			// packet cnt.
 
 // Value domain of pMacEntry->Sst
 typedef enum _Sst {
@@ -1435,36 +1429,6 @@ typedef enum _AuthState {
     AS_AUTHENTICATING   // STA is waiting for AUTH seq#3 using SHARED KEY
 } AUTH_STATE;
 
-//for-wpa value domain of pMacEntry->WpaState  802.1i D3   p.114
-typedef enum _ApWpaState {
-    AS_NOTUSE,              // 0
-    AS_DISCONNECT,          // 1
-    AS_DISCONNECTED,        // 2
-    AS_INITIALIZE,          // 3
-    AS_AUTHENTICATION,      // 4
-    AS_AUTHENTICATION2,     // 5
-    AS_INITPMK,             // 6
-    AS_INITPSK,             // 7
-    AS_PTKSTART,            // 8
-    AS_PTKINIT_NEGOTIATING, // 9
-    AS_PTKINITDONE,         // 10
-    AS_UPDATEKEYS,          // 11
-    AS_INTEGRITY_FAILURE,   // 12
-    AS_KEYUPDATE,           // 13
-} AP_WPA_STATE;
-
-// for-wpa value domain of pMacEntry->WpaState  802.1i D3   p.114
-typedef enum _GTKState {
-    REKEY_NEGOTIATING,
-    REKEY_ESTABLISHED,
-    KEYERROR,
-} GTK_STATE;
-
-//  for-wpa  value domain of pMacEntry->WpaState  802.1i D3   p.114
-typedef enum _WpaGTKState {
-    SETKEYS,
-    SETKEYS_DONE,
-} WPA_GTK_STATE;
 // ====================== end of AP mlme.h ============================
 
 
